/**
 *  Copyright (c) 1997-2013, tinygroup.org (luo_guo@live.cn).
 *
 *  Licensed under the GPL, Version 3.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *       http://www.gnu.org/licenses/gpl.html
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 * --------------------------------------------------------------------------
 *  版权 (c) 1997-2013, tinygroup.org (luo_guo@live.cn).
 *
 *  本开源软件遵循 GPL 3.0 协议;
 *  如果您不遵循此协议，则不被允许使用此文件。
 *  你可以从下面的地址获取完整的协议文本
 *
 *       http://www.gnu.org/licenses/gpl.html
 */
package org.tinygroup.bizframe;

import java.util.List;

/**
 * 权限检查器
 * 
 * @author luoguo
 * 
 */
@SuppressWarnings("rawtypes")
public interface PermissionManager {
	/**
	 * 设置权限主体检查器列表
	 * 
	 * @param permissionSubjectCheckerList
	 */
	void setPermissionSubjectCheckerList(
			List<PermissionSubjectChecker> permissionSubjectCheckerList);

	/**
	 * 设置权限主体检查器列表
	 * 
	 * @param permissionSubjectCheckerList
	 */
	void setPermissionObjectCheckerList(
			List<PermissionObjectChecker> permissionObjectCheckerList);

	/**
	 * 检查sourceObject是否属于destObject
	 * 
	 * @param sourceObject
	 * @param destObject
	 * @return
	 */
	boolean belongTo(PermissionSubject sourceObject,
			PermissionSubject destObject);

	/**
	 * 返回sourceObject是否包含了destObject
	 * 
	 * @param sourceObject
	 * @param destObject
	 * @return
	 */
	boolean contains(PermissionSubject sourceObject,
			PermissionSubject destObject);

	/**
	 * 检查sourceObject是否属于destObject
	 * 
	 * @param sourceObject
	 * @param destObject
	 * @return
	 */
	boolean belongTo(PermissionObject sourceObject, PermissionObject destObject);

	/**
	 * 返回sourceObject是否包含了destObject
	 * 
	 * @param sourceObject
	 * @param destObject
	 * @return
	 */
	boolean contains(PermissionObject sourceObject, PermissionObject destObject);

	/**
	 * 是否支持缓冲
	 * 
	 * @return
	 */
	boolean isCacheSupport();

	/**
	 * 设置是否支持缓冲
	 * 
	 * @param support
	 */
	void setCacheSupport(boolean support);

	/**
	 * 权限客体是否支持继承
	 * 
	 * @return
	 */
	boolean isPermissionObjectInheritSupport();

	/**
	 * 设置权限客体是否支持继承
	 * 
	 * @param support
	 */
	void setPermissionObjectInheritSupport(boolean support);

	/**
	 * 权限主体是否支持继承
	 * 
	 * @return
	 */

	boolean isPermissionSubjectInheritSupport();

	/**
	 * 设置权限主体是否支持继承
	 * 
	 * @param support
	 */
	void setPermissionSubjectInheritSupport(boolean support);

	/**
	 * 设置权限策略列表
	 * 
	 * @param permissionStrategyList
	 */
	void setPermissionStrategy(PermissionCheckStrategy permissionStrategy);

	/**
	 * 添加权限主体
	 * 
	 * @param permissionSubject
	 */
	void addPermissionSubject(PermissionSubject permissionSubject);

	/**
	 * 添加权限客户
	 * 
	 * @param permissionObject
	 */
	void addPermissionObject(PermissionObject permissionObject);

	/**
	 * 是否禁止
	 * 
	 * @param permissionSubject
	 * @param permissionObject
	 * @return
	 */
	boolean isBlock(PermissionSubject permissionSubject,
			PermissionObject permissionObject);

	/**
	 * 是否允许
	 * 
	 * @param permissionSubject
	 * @param permissionObject
	 * @return
	 */
	boolean isAllow(PermissionSubject permissionSubject,
			PermissionObject permissionObject);

	/**
	 * 是否禁止
	 * 
	 * @param permissionSubject
	 * @param permissionObject
	 * @return
	 */
	boolean isBlock(String permissionSubjectType, String permissionSubjectId,
			String permissionObjectType, String permissionObjectId);

	/**
	 * 是否允许
	 * 
	 * @param permissionSubject
	 * @param permissionObject
	 * @return
	 */
	boolean isAllow(String permissionSubjectType, String permissionSubjectId,
			String permissionObjectType, String permissionObjectId);

	/**
	 * 是否直接禁止
	 * 
	 * @param permissionSubject
	 * @param permissionObject
	 * @return
	 */
	boolean isBlockDirectly(PermissionSubject permissionSubject,
			PermissionObject permissionObject);

	/**
	 * 是否直接允许
	 * 
	 * @param permissionSubject
	 * @param permissionObject
	 * @return
	 */
	boolean isAllowDirectly(PermissionSubject permissionSubject,
			PermissionObject permissionObject);

	/**
	 * 是否直接禁止
	 * 
	 * @return
	 */
	boolean isBlockDirectly(String permissionSubjectType,
			String permissionSubjectId, String permissionObjectType,
			String permissionObjectId);

	/**
	 * 是否直接允许
	 * 
	 * @return
	 */
	boolean isAllowDirectly(String permissionSubjectType,
			String permissionSubjectId, String permissionObjectType,
			String permissionObjectId);

	/**
	 * 给权限主体添加权限客体
	 * 
	 * @param permissionSubject
	 * @param permissionObject
	 */
	void addAllowPermission(PermissionSubject permissionSubject,
			PermissionObject permissionObject);

	/**
	 * 给一组权限主体添加一组权限客体
	 * 
	 * @param permissionSubject
	 * @param permissionObject
	 */
	void addAllowPermission(List<PermissionSubject> permissionSubjectList,
			List<PermissionObject> permissionObjectList);

	/**
	 * 
	 * 给权限主体删除权限客体
	 * 
	 * @param permissionSubject
	 * @param permissionObject
	 */
	void removeAllowPermission(PermissionSubject permissionSubject,
			PermissionObject permissionObject);

	/**
	 * 给一组权限主体删除一组权限客体
	 * 
	 * @param permissionSubject
	 * @param permissionObject
	 */
	void removeAllowPermission(List<PermissionSubject> permissionSubjectList,
			List<PermissionObject> permissionObjectList);

	/**
	 * 给权限主体添加权限客体
	 * 
	 * @param permissionSubject
	 * @param permissionObject
	 */
	void addAllowPermission(String permissionSubjectType,
			String permissionSubjectId, String permissionObjectType,
			String permissionObjectId);

	/**
	 * 
	 * 给权限主体删除权限客体
	 * 
	 * @param permissionSubject
	 * @param permissionObject
	 */
	void removeAllowPermission(String permissionSubjectType,
			String permissionSubjectId, String permissionObjectType,
			String permissionObjectId);

	/**
	 * 给权限主体添加权限客体
	 * 
	 * @param permissionSubject
	 * @param permissionObject
	 */
	void addBlockPermission(PermissionSubject permissionSubject,
			PermissionObject permissionObject);

	/**
	 * 给一组权限主体添加一组权限客体
	 * 
	 * @param permissionSubject
	 * @param permissionObject
	 */
	void addBlockPermission(List<PermissionSubject> permissionSubjectList,
			List<PermissionObject> permissionObjectList);

	/**
	 * 
	 * 给权限主体删除权限客体
	 * 
	 * @param permissionSubject
	 * @param permissionObject
	 */
	void removeBlockPermission(PermissionSubject permissionSubject,
			PermissionObject permissionObject);

	/**
	 * 给一组权限主体删除一组权限客体
	 * 
	 * @param permissionSubject
	 * @param permissionObject
	 */
	void removeBlockPermission(List<PermissionSubject> permissionSubjectList,
			List<PermissionObject> permissionObjectList);

	/**
	 * 给权限主体添加权限客体
	 * 
	 * @param permissionSubject
	 * @param permissionObject
	 */
	void addBlockPermission(String permissionSubjectType,
			String permissionSubjectId, String permissionObjectType,
			String permissionObjectId);

	/**
	 * 
	 * 给权限主体删除权限客体
	 * 
	 * @param permissionSubject
	 * @param permissionObject
	 */
	void removeBlockPermission(String permissionSubjectType,
			String permissionSubjectId, String permissionObjectType,
			String permissionObjectId);

}
